package com.emall.flash.core.aop;

import com.emall.flash.bean.core.TenantId;
import com.emall.flash.bean.entity.BaseEntity;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.stereotype.Component;

import java.lang.reflect.Method;
import java.util.Objects;

/**
 * 商圈过滤
 *
 * @author fengshuonan
 * @date 2016年12月6日 下午8:48:30
 */
@Aspect
@Component
public class TenantAop {


    @Pointcut(value = "@annotation(com.emall.flash.bean.core.TenantId)")
    public void cutService() {
    }

    @Before("cutService()")
    public void recordSysLog(JoinPoint point) throws Throwable {
        handleData(point);
    }

    //查询指定元素的位置
    public static int search(String[] arr, String element) {
        for (int i = 0; i < arr.length; i++) {
            if (Objects.equals(arr[i], element)) {
                return i;
            }
        }

        return -1;
    }

    private void handleData(JoinPoint point) throws Exception {
        // 获得注解（可以获取注解上配置的参数，从而实现复杂逻辑）
        TenantId controllerDataScope = getAnnotationLog(point);
        if (controllerDataScope == null) {
            return;
        }
        // 获取当前的用户
        try {
            Object[] args = point.getArgs(); // 参数值
            String[] argNames = ((MethodSignature)point.getSignature()).getParameterNames(); // 参数名
            Long tenantId = null;
            int i = search(argNames,"tenantId");
            if (i != -1){
                tenantId = (Long) args[i];
            }
            if (tenantId != null) {
                dataScopeFilter(point, tenantId, controllerDataScope.dataAlias());
            }
        } catch (Exception e) {
            e.printStackTrace();
            return;
        }
    }

    /**
     * 数据范围过滤
     *
     * @param joinPoint 切点
     * @param user      用户
     * @param dataAlias 数据查询主表的别名
     */
    public static void dataScopeFilter(JoinPoint joinPoint, Long tenantId, String dataAlias) {
        BaseEntity baseEntity = (BaseEntity) joinPoint.getArgs()[0];
        baseEntity.setTenant(" AND " + dataAlias + ".tenant_id = " + tenantId);
    }


    /**
     * 是否存在注解，如果存在就获取
     */
    private TenantId getAnnotationLog(JoinPoint joinPoint) {
        Signature signature = joinPoint.getSignature();
        MethodSignature methodSignature = (MethodSignature) signature;
        Method method = methodSignature.getMethod();
        if (method != null) {
            return method.getAnnotation(TenantId.class);
        }
        return null;
    }
}
