package com.emall.flash.service.shop;


import com.emall.flash.bean.entity.shop.Category;
import com.emall.flash.bean.entity.shop.Goods;
import com.emall.flash.bean.vo.node.CategoryNode;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.dao.shop.CategoryRepository;
import com.emall.flash.service.BaseService;
import com.emall.flash.service.system.ManagerService;
import com.emall.flash.utils.Lists;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class CategoryService extends BaseService<Category,Long, CategoryRepository> {
    private Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private CategoryRepository categoryRepository;
    @Autowired
    private ManagerService managerService;
    @Autowired
    private GoodsService goodsService;

    public List<CategoryNode> getCategories(Long tenantId, boolean ifFiltration) {
        List<SearchFilter> filters = Lists.newArrayList();
        filters.add(SearchFilter.build("delFlag", SearchFilter.Operator.EQ,0));
        if (tenantId!=null)
            filters.add(SearchFilter.build("tenantId", SearchFilter.Operator.EQ,tenantId));
        List<Category> list = queryAll(filters,Sort.by(Sort.Direction.ASC,"sort"));
        List<CategoryNode> nodes = Lists.newArrayList();
        for(Category category:list){
            if(category.getPid()==null) {
                CategoryNode node = new CategoryNode();
                BeanUtils.copyProperties(category, node);
                nodes.add(node);
            }

        }
        for(CategoryNode node:nodes){
            for(Category category:list){
                if(category.getPid()!=null&&category.getPid().intValue() == node.getId().intValue()){
                    if (ifFiltration){
                        List<SearchFilter> filterList = Lists.newArrayList();
                        filterList.add(SearchFilter.build("idCategory", category.getId()));
                        if (tenantId!=null)
                            filterList.add(SearchFilter.build("tenantId", tenantId));
                        filterList.add(SearchFilter.build("isOnSale", true));
                        List<Goods> goods = goodsService.queryAll(filterList);
                        if (!goods.isEmpty()) {
                            CategoryNode child = new CategoryNode();
                            BeanUtils.copyProperties(category, child);
                            if (node.getChildren() == null) {
                                node.setChildren(Lists.newArrayList());
                            }
                            node.getChildren().add(child);
                        }
                    } else {
                        CategoryNode child = new CategoryNode();
                        BeanUtils.copyProperties(category, child);
                        if (node.getChildren() == null) {
                            node.setChildren(Lists.newArrayList());
                        }
                        node.getChildren().add(child);
                    }
                }
            }
        }
        List<CategoryNode> nodeList = Lists.newArrayList();
        for(CategoryNode node:nodes){
            if (node.getChildren()!=null&&!node.getChildren().isEmpty())
                nodeList.add(node);
        }
        return nodeList;
    }
}

