package com.emall.flash.service.shop;


import com.emall.flash.bean.entity.shop.Goods;
import com.emall.flash.bean.entity.shop.GoodsSku;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.bean.vo.shop.GoodsVo;
import com.emall.flash.cache.CacheDao;
import com.emall.flash.dao.shop.GoodsRepository;
import com.emall.flash.dao.system.FileInfoRepository;
import com.emall.flash.service.BaseService;
import com.emall.flash.utils.Lists;
import com.emall.flash.utils.RedisCons;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.concurrent.TimeUnit;

@Service
public class GoodsService extends BaseService<Goods,Long, GoodsRepository> {
    private Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private GoodsRepository goodsRepository;
    @Autowired
    private GoodsSkuService goodsSkuService;
    @Autowired
    private FileInfoRepository fileInfoRepository;
    @Autowired
    private CacheDao cacheDao;
    @Autowired
    private RedissonClient redissonClient;

    @Override
    public void deleteById(Long id) {
        Goods goods = get(id);
        goods.setIsDelete(true);
        goods.setDelFlag(1);
        update(goods);
    }
    public GoodsVo getDetail(Long id){
        Goods goods = get(id);
        List<GoodsSku> skuList = goodsSkuService.queryAll(Lists.newArrayList(
                SearchFilter.build("idGoods",id),
                SearchFilter.build("isDeleted",false)
        ));
        GoodsVo vo = new GoodsVo();
        vo.setGoods(goods);
        vo.setSkuList(skuList);
        return vo;
    }

    /**
     * 商品上架或者下架
     * @param id
     * @param isOnSale
     */
    public void changeIsOnSale(Long id, Boolean isOnSale) {
        Goods goods = get(id);
        //redis存库存 单规格：【goods:count:${gid}】 多规格【goodsSku:count:${gsid}】
        List<GoodsSku> skuList = goodsSkuService.queryAll(Lists.newArrayList(
                SearchFilter.build("idGoods", id),
                SearchFilter.build("isDeleted", false)
        ));
        if (isOnSale.equals(true)){
            goods.setIsOnSale(isOnSale);
            update(goods);
            if (skuList.isEmpty()){
                cacheDao.hset(RedisCons.GOODS_COUNT , id,goods.getStock());
            } else {
                for (GoodsSku goodsSku : skuList) {
                    cacheDao.hset(RedisCons.GOODS_SKU_COUNT, goodsSku.getId(), goodsSku.getStock());
                }
                cacheDao.hset(RedisCons.GOODS_COUNT , id,goods.getStock());
            }
        } else {
            //如果取消发布，加redis锁
            boolean isGetLock = false;
            RLock lock = redissonClient.getLock(RedisCons.GOODS_INFO + id);
            try {
                isGetLock = lock.tryLock(Long.parseLong(RedisCons.ORDER_INFO_LOCK_WAIT_TIME),
                        Long.parseLong(RedisCons.ORDER_INFO_LOCK_LEASE_TIME), TimeUnit.SECONDS);
                if (isGetLock) {
                    goods.setIsOnSale(isOnSale);
                    Integer count = cacheDao.hget(RedisCons.GOODS_COUNT ,id,Integer.class);
                    goods.setStock(count);
                    update(goods);
                    if (skuList.isEmpty()){
                        cacheDao.hdel(RedisCons.GOODS_COUNT , id);
                    } else {
                        for (GoodsSku goodsSku : skuList) {
                            Integer skuCount = cacheDao.hget(RedisCons.GOODS_SKU_COUNT ,goodsSku.getId(),Integer.class);
                            goodsSku.setStock(skuCount);
                            goodsSkuService.update(goodsSku);
                            cacheDao.hdel(RedisCons.GOODS_SKU_COUNT, goodsSku.getId());
                        }
                        cacheDao.hdel(RedisCons.GOODS_COUNT , id);
                    }
                }
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
                isGetLock = false;
            } finally {
                lock.unlock();
                logger.error("释放成功");
            }
        }
    }
}

