package com.emall.flash.service.shop;


import com.emall.flash.bean.entity.shop.Tenant;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.dao.shop.TenantRepository;
import com.emall.flash.service.BaseService;
import com.emall.flash.utils.Lists;
import com.emall.flash.utils.factory.Page;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;

import java.util.Map;

@Service
public class TenantService extends BaseService<Tenant,Long, TenantRepository> {
    @Autowired
    private TenantRepository tenantRepository;

    /**
     * 校验名称是否重复
     * @param tenant
     * @return
     */
    public Boolean isRepeat(Tenant tenant) {
        Tenant old = null;
        if (tenant.getId() != null) {
            old = get(
                    Lists.newArrayList(SearchFilter.build("name", tenant.getName()),
                            SearchFilter.build("delFlag", 0),
                            SearchFilter.build("id", SearchFilter.Operator.NE, tenant.getId())
                    ));
        } else {
            old = get( Lists.newArrayList(SearchFilter.build("name", tenant.getName()),
                    SearchFilter.build("delFlag", 0)));
        }
        return old != null;
    }

    /**
     * 校验是否管理员已经绑定商圈
     * @param tenant
     * @return
     */
    public Boolean isBind(Tenant tenant) {
        Tenant old = null;
        if (tenant.getId() != null) {
            old = get(
                    Lists.newArrayList(SearchFilter.build("manager", tenant.getManager()),
                            SearchFilter.build("delFlag", 0),
                            SearchFilter.build("id", SearchFilter.Operator.NE, tenant.getId())
                    ));
        } else {
            old = get(
                    Lists.newArrayList(SearchFilter.build("manager", tenant.getManager()),
                    SearchFilter.build("delFlag", 0)
                    ));
        }
        return old != null;
    }

    public Page selectPage(Page page, Long cityId, String name) {
        Pageable pageable = null;
        if (page.getSort() != null) {
            pageable = PageRequest.of(page.getCurrent() - 1, page.getSize(), page.getSort());
        } else {
            pageable = PageRequest.of(page.getCurrent() - 1, page.getSize(), Sort.Direction.DESC, "id");
        }
        org.springframework.data.domain.Page<Map> list = tenantRepository.selectPage(cityId,name,pageable);
        page.setTotal(Integer.valueOf(list.getTotalElements() + ""));
        page.setRecords(list.getContent());
        return page;
    }

    public Page getShopRoleUser(Page page, String name) {
        Pageable pageable = null;
        if (page.getSort() != null) {
            pageable = PageRequest.of(page.getCurrent() - 1, page.getSize(), page.getSort());
        } else {
            pageable = PageRequest.of(page.getCurrent() - 1, page.getSize(), Sort.Direction.DESC, "id");
        }
        org.springframework.data.domain.Page<Map> list = tenantRepository.getShopRoleUser(name,pageable);
        page.setTotal(Integer.valueOf(list.getTotalElements() + ""));
        page.setRecords(list.getContent());
        return page;
    }
}

