package com.emall.flash.service.system;

import com.emall.flash.bean.entity.system.Cfg;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.cache.ConfigCache;
import com.emall.flash.dao.system.CfgRepository;
import com.emall.flash.service.BaseService;
import com.emall.flash.utils.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * CfgService
 *
 * @author enilu
 * @version 2018/11/17 0017
 */

@Service
@Transactional
public class CfgService extends BaseService<Cfg, Long, CfgRepository> {
    @Autowired
    private ConfigCache configCache;

    /**
     * 切记更新配置的时候调用该方法，而不要调用其父类的insert或者update方法，该方法会连同缓存一起更新
     *
     * @param cfg
     * @return
     */
    public Cfg saveOrUpdate(Cfg cfg) {
        if (cfg.getId() == null) {
            insert(cfg);
        } else {
            update(cfg);
        }
        configCache.cache();
        return cfg;
    }


    public Cfg getByCfgName(String cfgName) {
        List<SearchFilter> filters = Lists.newArrayList();
        filters.add(SearchFilter.build("delFlag", 0));
        filters.add(SearchFilter.build("cfgName", cfgName));
        return get(filters);
    }

    /**
     * 根据参数名获取参数值
     * 系统获取参数值统一使用该方法
     * 如果参数无法做到后台管理系统和用户端系统同步，这里建议直接从数据库获取
     * todo 建议生产中使用redis来统一管理该参数，这里从redis缓存中获取
     *
     * @param cfgName
     * @return
     */
    public String getCfgValue(String cfgName) {
//        return configCache.get(cfgName);
        return getByCfgName(cfgName).getCfgValue();
    }

    public void update(String cfgName, String cfgValue) {
        Cfg cfg = getByCfgName(cfgName);
        cfg.setCfgValue(cfgValue);
        update(cfg);
    }

    /**
     * 判断当前配置与数据库已有配置的参数名是否重复
     *
     * @param cfg
     * @return
     */
    public Boolean isRepeat(Cfg cfg) {
        Cfg old = null;
        if (cfg.getId() != null) {
            old = get(
                    Lists.newArrayList(SearchFilter.build("cfgName", cfg.getCfgName()),
                            SearchFilter.build("id", SearchFilter.Operator.NE, cfg.getId()),
                            SearchFilter.build("delFlag", 0)
                    ));
        } else {
            old = get(Lists.newArrayList(SearchFilter.build("cfgName", cfg.getCfgName()),
                    SearchFilter.build("delFlag", 0)));
        }
        return old != null;
    }
}
