package com.emall.flash.service.system;

import com.emall.flash.bean.entity.system.Dept;
import com.emall.flash.bean.entity.system.User;
import com.emall.flash.bean.vo.node.DeptNode;
import com.emall.flash.bean.vo.node.ZTreeNode;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.dao.system.DeptRepository;
import com.emall.flash.security.JwtUtil;
import com.emall.flash.service.BaseService;
import com.emall.flash.utils.Lists;
import com.google.common.base.Strings;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * Created  on 2018/3/21 0021.
 *
 * @author enilu
 */
@Service
public class DeptService extends BaseService<Dept,Long, DeptRepository> {
    @Autowired
    private DeptRepository deptRepository;
    @Autowired
    private ManagerService managerService;

    public List<ZTreeNode> tree() {
        String sql = "SELECT id, pid AS pId, simplename AS NAME, ( CASE WHEN (pId = 0 OR pId IS NULL) THEN 'true' ELSE 'false' END ) AS open FROM t_sys_dept";
        List nodes = deptRepository.queryObjBySql(sql, ZTreeNode.class);
        return nodes;
    }


    public List<Dept> query(String condition) {
        List<Dept> list = null;
        if (Strings.isNullOrEmpty(condition)) {
            list = deptRepository.findAll();
        } else {
            condition = "%" + condition + "%";
            list = deptRepository.findBySimplenameLikeOrFullnameLike(condition, condition);
        }
        return list;
    }

    public void deleteDept(Long deptId) {
        Dept dept = get(deptId);
        List<Dept> subDepts = deptRepository.findByPidsLike("%[" + dept.getId() + "]%");
        deptRepository.deleteAll(subDepts);
        deptRepository.delete(dept);
    }

    public List<DeptNode> queryAllNode(Long tenantId) {
        User loginUser = managerService.get(JwtUtil.getUserId());
        List<SearchFilter> filters = Lists.newArrayList();
        filters.add(SearchFilter.build("delFlag", 0));
        if (loginUser.getTenantId()!=null)
            filters.add(SearchFilter.build("tenantId", SearchFilter.Operator.EQ, loginUser.getTenantId()));
        else if (tenantId!= null)
            filters.add(SearchFilter.build("tenantId", SearchFilter.Operator.EQ, tenantId));
        else
            filters.add(SearchFilter.build("tenantId", SearchFilter.Operator.ISNULL));
        List<Dept> list = queryAll(filters);
        return generateTree(list);
    }

    public void deptSetPids(Dept dept) {
        if (dept.getPid() == null || dept.getPid().intValue() == 0) {
            dept.setPid(0L);
            dept.setPids("[0],");
        } else {
            Long pid = dept.getPid();
            Dept temp = get(pid);
            String pids = "";
            if (temp != null) {
                pids = temp.getPids();
            }
            dept.setPid(pid);
            dept.setPids(pids + "[" + pid + "],");
        }
    }

    private List<DeptNode> generateTree(List<Dept> list) {

        List<DeptNode> nodes = new ArrayList<>(20);
        for (Dept dept : list) {
            DeptNode deptNode = new DeptNode();
            BeanUtils.copyProperties(dept, deptNode);
            nodes.add(deptNode);
        }
        for (DeptNode deptNode : nodes) {
            for (DeptNode child : nodes) {
                if (child.getPid().intValue() == deptNode.getId().intValue()) {
                    List<DeptNode> children = deptNode.getChildren();
                    if (children == null) {
                        children = Lists.newArrayList();
                        deptNode.setChildren(children);
                    }
                    children.add(child);
                }
            }
        }
        List<DeptNode> result = new ArrayList<>(20);
        for (DeptNode node : nodes) {
            if (node.getPid().intValue() == 0) {
                result.add(node);
            }
        }
        return result;


    }



}
