package com.emall.flash.utils;

import com.emall.flash.bean.exception.ApplicationException;
import com.emall.flash.bean.exception.ApplicationExceptionEnum;
import com.google.zxing.BarcodeFormat;
import com.google.zxing.EncodeHintType;
import com.google.zxing.MultiFormatWriter;
import com.google.zxing.common.BitMatrix;
import com.google.zxing.qrcode.decoder.ErrorCorrectionLevel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.*;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

public class CyQrCodeUtil {
    private static final Logger log = LoggerFactory.getLogger(CyQrCodeUtil.class);
    private static final Integer QC_CODE_WIDTH = 235;
    private static final Integer QC_CODE_HEIGHT = 235;

    public CyQrCodeUtil() {
    }
    public static BufferedImage qRCodeCommon(String content) throws UnsupportedEncodingException {
        Qrcode qrcode = new Qrcode();
        qrcode.setQrcodeEncodeMode('B');
        qrcode.setQrcodeVersion(15);
        qrcode.setQrcodeErrorCorrect('M');
        BufferedImage image = new BufferedImage(QC_CODE_WIDTH, QC_CODE_HEIGHT, 1);
        Graphics2D gs = image.createGraphics();
        gs.setBackground(Color.white);
        gs.setColor(Color.black);
        gs.clearRect(0, 0, QC_CODE_WIDTH, QC_CODE_HEIGHT);
        Object var4 = null;

        try {
            byte[] codeOut = content.getBytes("utf-8");
            boolean[][] code = qrcode.calQrcode(codeOut);

            for(int i = 0; i < code.length; ++i) {
                for(int j = 0; j < code.length; ++j) {
                    if (code[j][i]) {
                        gs.fillRect(j * 3 + 2, i * 3 + 2, 3, 3);
                    }
                }
            }

            return image;
        } catch (Exception var8) {
            log.error(var8.getMessage());
            throw var8;
        }
    }


    public static InputStream getCode2(String urlCode,  int qrWidth, int qrHeight){
        ByteArrayInputStream stream = null;
        //需要编码的内容
        String png_base64 = null;
        //返回二维码 并且构造支付包web支付对象
        ByteArrayOutputStream baOp = new ByteArrayOutputStream();
        try {
            BufferedImage bufImg = createImage(urlCode,qrWidth,qrHeight);
            // 生成二维码QRCode图片
            ImageIO.write(bufImg, "png", baOp);
            byte[] bytes = baOp.toByteArray();//转换成字节
            stream = new ByteArrayInputStream(bytes);
            File file = new File("D:/3.png");
//            if (!file.getParentFile().exists()){
//                //文件夹不存在 生成
//                file.getParentFile().mkdirs();
//            }
            FileOutputStream fos = null;
            BufferedOutputStream bos = null;
            fos = new FileOutputStream(file);
            fos.write(bytes);
            bos = new BufferedOutputStream(fos);
            bos.write(bytes);
        } catch (Exception e) {
            log.error(e.getMessage());
            throw new ApplicationException(ApplicationExceptionEnum.QRCODE_ERROR);
        }
        return stream;
    }
    private static final String CHARSET = "utf-8";

    public static BufferedImage createImage(String content, int qrWidth, int qrHeight) throws Exception {
        Map<EncodeHintType, Object> hints = new Hashtable<>();
        hints.put(EncodeHintType.ERROR_CORRECTION, ErrorCorrectionLevel.H);
        hints.put(EncodeHintType.CHARACTER_SET, CHARSET);
        hints.put(EncodeHintType.MARGIN, 1);
        BitMatrix bitMatrix = new MultiFormatWriter().encode(content, BarcodeFormat.QR_CODE, qrWidth, qrHeight,hints);
        qrWidth = bitMatrix.getWidth();
        qrHeight = bitMatrix.getHeight();
        //创建二维码
        BufferedImage image = new BufferedImage(qrWidth, qrHeight, BufferedImage.TYPE_INT_RGB);
        for (int x = 0; x < qrWidth; x++) {
            for (int y = 0; y < qrHeight; y++) {
                image.setRGB(x, y, bitMatrix.get(x, y) ? 0xFF000000 : 0xFFFFFFFF);
            }
        }
        //在二维码下方增加文字显示
        BufferedImage bi = new BufferedImage(qrWidth, qrHeight, BufferedImage.TYPE_INT_RGB);
        Graphics2D g2 = bi.createGraphics();
        g2.setBackground(new Color(0xFF,0xFF,0xFF));
        //通过使用当前绘图表面的背景色进行填充来清除指定的矩形。
        g2.clearRect(0, 0, qrWidth, qrHeight);
        g2.drawImage(image, 0, 0, qrWidth, qrHeight, null); //x，y为image图片的位置
        image=bi;
        return image;
    }
}
