package com.emall.flash.utils;


import org.gavaghan.geodesy.Ellipsoid;
import org.gavaghan.geodesy.GeodeticCalculator;
import org.gavaghan.geodesy.GeodeticCurve;
import org.gavaghan.geodesy.GlobalCoordinates;

import java.math.BigDecimal;

public class GeoUtil {

    /**
     * 根据经纬度，计算两点间的距离
     *
     * @param longitudeOne 第一个点的经度
     * @param latitudeOne  第一个点的纬度
     * @param longitudeTwo 第二个点的经度
     * @param latitudeTwo  第二个点的纬度
     * @return 返回距离，单位m
     */
    public static double getDistance(double longitudeOne, double latitudeOne, double longitudeTwo, double latitudeTwo) {
        //创建GeodeticCalculator，调用计算方法，传入坐标系、经纬度用于计算距离
        GeodeticCurve geoCurve = new GeodeticCalculator().calculateGeodeticCurve(
                Ellipsoid.Sphere,
                new GlobalCoordinates(latitudeOne, longitudeOne),
                new GlobalCoordinates(latitudeTwo,longitudeTwo)
        );
        double distance = geoCurve.getEllipsoidalDistance();
        return new BigDecimal(distance).divide(new BigDecimal(1000), 2, BigDecimal.ROUND_HALF_UP).doubleValue();
    }


}
